@@ -8,12 +8,15 @@ import six
 
 from saml2 import saml
 from saml2 import xmlenc
-from saml2.attribute_converter import from_local, get_local_name
+from saml2.attribute_converter import from_local, ac_factory
+from saml2.attribute_converter import get_local_name
 from saml2.s_utils import assertion_factory
 from saml2.s_utils import factory
-from saml2.s_utils import sid, MissingValue
+from saml2.s_utils import sid
+from saml2.s_utils import MissingValue
 from saml2.saml import NAME_FORMAT_URI
-from saml2.time_util import instant, in_a_while
+from saml2.time_util import instant
+from saml2.time_util import in_a_while
 
 logger = logging.getLogger(__name__)
 
@@ -78,19 +81,29 @@ def filter_on_attributes(ava, required=None, optional=None, acs=None,
     """
 
     def _match_attr_name(attr, ava):
-        try:
-            friendly_name = attr["friendly_name"]
-        except KeyError:
-            friendly_name = get_local_name(acs, attr["name"],
-                                           attr["name_format"])
+        local_name = None
+
+        for a in ['name_format', 'friendly_name']:
+            _val = attr.get(a)
+            if _val:
+                if a == 'name_format':
+                    local_name = get_local_name(acs, attr['name'], _val)
+                else:
+                    local_name = _val
+                break
+
+        if local_name:
+            _fn = _match(local_name, ava)
+        else:
+            _fn = None
 
-        _fn = _match(friendly_name, ava)
         if not _fn:  # In the unlikely case that someone has provided us with
             #  URIs as attribute names
             _fn = _match(attr["name"], ava)
 
         return _fn
 
+
     def _apply_attr_value_restrictions(attr, res, must=False):
         try:
             values = [av["text"] for av in attr["attribute_value"]]
@@ -105,7 +118,6 @@ def filter_on_attributes(ava, required=None, optional=None, acs=None,
         return _filter_values(ava[_fn], values, must)
 
     res = {}
-
     if required is None:
         required = []
 
@@ -115,8 +127,7 @@ def filter_on_attributes(ava, required=None, optional=None, acs=None,
         if _fn:
             _apply_attr_value_restrictions(attr, res, True)
         elif fail_on_unfulfilled_requirements:
-            desc = "Required attribute missing: '%s' (%s)" % (attr["name"],
-                                                              _fn)
+            desc = "Required attribute missing: '%s'" % (attr["name"])
             raise MissingValue(desc)
 
     if optional is None:
@@ -285,9 +296,7 @@ def post_entity_categories(maps, **kwargs):
                         else:
                             attrs = atlist
                         for _key in key:
-                            try:
-                                assert _key in ecs
-                            except AssertionError:
+                            if _key not in ecs:
                                 attrs = []
                                 break
                     elif key in ecs:
@@ -342,8 +351,11 @@ class Policy(object):
             else:
                 ecs = []
                 for cat in items:
-                    _mod = importlib.import_module(
-                        "saml2.entity_category.%s" % cat)
+                    try:
+                        _mod = importlib.import_module(cat)
+                    except ImportError:
+                        _mod = importlib.import_module(
+                            "saml2.entity_category.%s" % cat)
                     _ec = {}
                     for key, items in _mod.RELEASE.items():
                         alist = [k.lower() for k in items]
@@ -500,6 +512,9 @@ class Policy(object):
 
         _ava = None
 
+        if not self.acs:  # acs MUST have a value, fall back to default.
+            self.acs = ac_factory()
+
         _rest = self.get_entity_categories(sp_entity_id, mdstore, required)
         if _rest:
             _ava = filter_attribute_value_assertions(ava.copy(), _rest)
