@@ -7,14 +7,16 @@ import logging.handlers
 import os
 import re
 import sys
+from logging.config import dictConfig as configure_logging_by_dict
 
 import six
 
-from saml2 import root_logger, BINDING_URI, SAMLError
-from saml2 import BINDING_SOAP
-from saml2 import BINDING_HTTP_REDIRECT
-from saml2 import BINDING_HTTP_POST
 from saml2 import BINDING_HTTP_ARTIFACT
+from saml2 import BINDING_HTTP_POST
+from saml2 import BINDING_HTTP_REDIRECT
+from saml2 import BINDING_SOAP
+from saml2 import BINDING_URI
+from saml2 import SAMLError
 
 from saml2.attribute_converter import ac_factory
 from saml2.assertion import Policy
@@ -22,12 +24,14 @@ from saml2.mdstore import MetadataStore
 from saml2.saml import NAME_FORMAT_URI
 from saml2.virtual_org import VirtualOrg
 
+
 logger = logging.getLogger(__name__)
 
 __author__ = 'rolandh'
 
 
 COMMON_ARGS = [
+    "logging",
     "debug",
     "entityid",
     "xmlsec_binary",
@@ -47,12 +51,12 @@ COMMON_ARGS = [
     "contact_person",
     "name_form",
     "virtual_organization",
-    "logger",
     "only_use_keys_in_metadata",
     "disable_ssl_certificate_validation",
     "preferred_binding",
     "session_storage",
     "assurance_certification",
+    "entity_attributes",
     "entity_category",
     "entity_category_support",
     "xmlsec_path",
@@ -68,7 +72,6 @@ COMMON_ARGS = [
     "extensions",
     "allow_unknown_attributes",
     "crypto_backend",
-    "id_attr_name",
     "delete_tmpfiles",
 ]
 
@@ -89,6 +92,7 @@ SP_ARGS = [
     "allow_unsolicited",
     "ecp",
     "name_id_format",
+    "name_id_policy_format",
     "name_id_format_allow_create",
     "logout_requests_signed",
     "requested_attribute_name_format",
@@ -139,24 +143,6 @@ SPEC = {
     "aq": COMMON_ARGS + COMPLEX_ARGS + AQ_ARGS,
 }
 
-# --------------- Logging stuff ---------------
-
-LOG_LEVEL = {
-    'debug': logging.DEBUG,
-    'info': logging.INFO,
-    'warning': logging.WARNING,
-    'error': logging.ERROR,
-    'critical': logging.CRITICAL}
-
-LOG_HANDLER = {
-    "rotating": logging.handlers.RotatingFileHandler,
-    "syslog": logging.handlers.SysLogHandler,
-    "timerotate": logging.handlers.TimedRotatingFileHandler,
-    "memory": logging.handlers.MemoryHandler,
-}
-
-LOG_FORMAT = "%(asctime)s %(name)s:%(levelname)s %(message)s"
-
 _RPA = [BINDING_HTTP_REDIRECT, BINDING_HTTP_POST, BINDING_HTTP_ARTIFACT]
 _PRA = [BINDING_HTTP_POST, BINDING_HTTP_REDIRECT, BINDING_HTTP_ARTIFACT]
 _SRPA = [BINDING_SOAP, BINDING_HTTP_REDIRECT, BINDING_HTTP_POST,
@@ -188,6 +174,7 @@ class Config(object):
     def_context = ""
 
     def __init__(self, homedir="."):
+        self.logging = None
         self._homedir = homedir
         self.entityid = None
         self.xmlsec_binary = None
@@ -209,9 +196,9 @@ class Config(object):
         self.contact_person = None
         self.name_form = None
         self.name_id_format = None
+        self.name_id_policy_format = None
         self.name_id_format_allow_create = None
         self.virtual_organization = None
-        self.logger = None
         self.only_use_keys_in_metadata = True
         self.logout_requests_signed = None
         self.disable_ssl_certificate_validation = None
@@ -225,10 +212,10 @@ class Config(object):
         self.domain = ""
         self.name_qualifier = ""
         self.assurance_certification = []
+        self.entity_attributes = []
         self.entity_category = []
         self.entity_category_support = []
         self.crypto_backend = 'xmlsec1'
-        self.id_attr_name = None
         self.scope = ""
         self.allow_unknown_attributes = False
         self.extension_schema = {}
@@ -360,6 +347,9 @@ class Config(object):
             except TypeError:  # Something that can't be a string
                 setattr(self, arg, cnf[arg])
 
+        if self.logging is not None:
+            configure_logging_by_dict(self.logging)
+
         if not self.delete_tmpfiles:
             logger.warning(
                 "delete_tmpfiles is set to False; "
@@ -453,63 +443,6 @@ class Config(object):
         else:
             return unspec
 
-    def log_handler(self):
-        try:
-            _logconf = self.logger
-        except KeyError:
-            return None
-
-        handler = None
-        for htyp in LOG_HANDLER:
-            if htyp in _logconf:
-                if htyp == "syslog":
-                    args = _logconf[htyp]
-                    if "socktype" in args:
-                        import socket
-                        if args["socktype"] == "dgram":
-                            args["socktype"] = socket.SOCK_DGRAM
-                        elif args["socktype"] == "stream":
-                            args["socktype"] = socket.SOCK_STREAM
-                        else:
-                            raise ConfigurationError("Unknown socktype!")
-                    try:
-                        handler = LOG_HANDLER[htyp](**args)
-                    except TypeError:  # difference between 2.6 and 2.7
-                        del args["socktype"]
-                        handler = LOG_HANDLER[htyp](**args)
-                else:
-                    handler = LOG_HANDLER[htyp](**_logconf[htyp])
-                break
-
-        if handler is None:
-            # default if rotating logger
-            handler = LOG_HANDLER["rotating"]()
-
-        if "format" in _logconf:
-            formatter = logging.Formatter(_logconf["format"])
-        else:
-            formatter = logging.Formatter(LOG_FORMAT)
-
-        handler.setFormatter(formatter)
-        return handler
-
-    def setup_logger(self):
-        if root_logger.level != logging.NOTSET:  # Someone got there before me
-            return root_logger
-
-        _logconf = self.logger
-        if _logconf is None:
-            return root_logger
-
-        try:
-            root_logger.setLevel(LOG_LEVEL[_logconf["loglevel"].lower()])
-        except KeyError:  # reasonable default
-            root_logger.setLevel(logging.INFO)
-
-        root_logger.addHandler(self.log_handler())
-        root_logger.info("Logging started")
-        return root_logger
-
     def endpoint2service(self, endpoint, context=None):
         endps = self.getattr("endpoints", context)
 
